*---------------------------------------------------------------------
*-- This program is meant to be run from the project's main directory ...
*-- the following directories must exist as subdirectories of the
*-- project directory:
*--    1.  LIBS     - contains application class libraries
*--    2.  SECURITY - contains TABLE2.DBF
*--
*-- Its purpose is to extract the security labels and descriptions
*-- from the application class libraries and place them into TABLE2.
*-- TABLE2's purpose in life is to identify every control in the 
*-- application that can have security assigned to it.
*--
*-- Inputs:
*--   Controls with values present in the cSecurityLabel and 
*--   cDescriptiveName properties set
*-- 
*-- Collaborating Objects
*--   1.  cData.VCX
*--       cString::Encrypt()
*---------------------------------------------------------------------
LPARAMETERS tcQualifier

LOCAL llRetVal, ;
      lcProjectFileAlias, ;
      lcCount

CLOSE ALL

#INCLUDE "..\INCLUDE\APPINCL.H"

SET PROCEDURE TO COMMONPATH\progs\setup.prg ADDITIVE

llRetVal = .F.

IF PCOUNT() = 0      
   lcProjectFileAlias = ""

	llRetVal = GetProjectFile( @lcProjectFileAlias )
	llRetVal = llRetVal AND GetClassLibraryNamesFromProject( lcProjectFileAlias )
	llRetVal = llRetVal AND ProcessSecurityLabels()
	llRetVal = llRetVal AND SychronizeSecurityTables()
ELSE
	IF TYPE('tcQualifier') == "C" AND .NOT. EMPTY( tcQualifier )
		DO CASE
		CASE UPPER( tcQualifier ) = "REMOVE_BAD_SECURITY_LABELS"
			llRetVal = RemoveBadSecurityLabels()
		OTHERWISE
		ENDCASE
	ENDIF
ENDIF

RETURN llRetVal

*============================================
FUNCTION GetProjectFile( tcProjectFileAlias )
*============================================
   LOCAL llRetVal , ;
         lcProjectFile
   
   lcProjectFile = GETFILE( "PJX", "Project:" )

   IF .NOT. EMPTY( lcProjectFile ) AND ;
      FILE( lcProjectFile )
      llRetVal = .T.
   ELSE
      llRetVal = .F.
   ENDIF
   
   IF llRetVal
      SELECT 0
      USE( lcProjectFile ) ALIAS project
   ENDIF
   
   IF USED( 'project')
      tcProjectFileAlias = "project"
   ELSE
      tcProjectFileAlias = ""
      llRetVal = .F.
   ENDIF
   
   RETURN llRetVal    

ENDFUNC

*=========================================================
FUNCTION GetClassLibraryNamesFromProject( tcProjectAlias )
*=========================================================
   LOCAL llRetVal
   
   SELECT name FROM ( tcProjectAlias )      ;
    WHERE Type = "V"                    AND ;
          .NOT. ("CDATA" $ UPPER(name) )    ;
     INTO CURSOR crsVCX
     
   IF USED( 'crsVCX') AND RECCOUNT( 'crsVCX' ) > 0
      llRetVal = .T.
   ELSE
      llRetVal = .F.
   ENDIF
  
   IF USED( tcProjectAlias )
      USE IN ( tcProjectAlias )
   ENDIF
  
   RETURN llRetVal     
                
ENDFUNC

*===============================
FUNCTION ProcessSecurityLabels()
*===============================

   SELECT crsVCX

   SCAN
      WAIT WINDOW "Extracting security labels from ... " + ;
                   ALLTRIM(UPPER( crsVCX.name )) NOWAIT
      =ExtractSecurityLabels( crsVCX.name )
   ENDSCAN
   
   WAIT WINDOW "Extracting security labels from REPOLIST !!! " NOWAIT
   
   IF USED( 'crsVCX' )
      USE IN crsVCX
   ENDIF
   
   =ExtractReportSecurityLabels()
   WAIT WINDOW "Security label extraction complete !!! " TIMEOUT 1
   
ENDFUNC

*==========================================================
FUNCTION ExtractSecurityLabels( tcApplicationClassLibrary )
*==========================================================
   LOCAL lcOldPath, ;
         lnOldWA
         
   lcOldPath = SET('PATH')
   SET PATH TO security
   lnOldWA = SELECT()
   
   SELECT 0
   USE ( tcApplicationClassLibrary ) ALIAS controls NOUPDATE

   SELECT 0
   USE security\table2

   SELECT controls

   SET CLASSLIB TO COMMONPATH\libs\cdata.vcx
   oString = CREATEOBJ('cstring')

   *----------------------------------------------------------------
   *-- Scan through each control in the ACNTRL.VCX looking for class 
   *-- definitions that define a security control.  A security control
   *-- is defined as any control THAT HAS A VALUE placed into its
   *-- ==> "cSecurityLabel" <== property.
   *-----------------------------------------------------------------
   SCAN

      SCATTER MEMO MEMVAR 
      *------------------------------------------------
      *-- Get the properties associated with the object
      *------------------------------------------------
      lnPropLength = LEN(m.properties)

      lnNumberOfSecurityLabels = OCCURS( UPPER("cSecurityLabel"), ;
                                         UPPER(m.properties) )
      FOR lnCount = 1 to lnNumberOfSecurityLabels
                                     
         lnSecurityLabelPropertyNameStart = AT( UPPER("cSecurityLabel"), ;
                                                UPPER(m.properties), ;
                                                lnCount )
   
         IF lnSecurityLabelPropertyNameStart > 0
            *----------------------------------------------------------------------
            *-- Obtain a string that begins with the property name "cSecurityLabel"
            *---------------------------------------------------------------------- 
            lcSecurityLabelValue = SUBSTR( m.properties, ;
                                           lnSecurityLabelPropertyNameStart, ;
                                           87)

            *---------------------------------------------------
            *-- Extract the value of the cSecurityLabel property
            *---------------------------------------------------                                
            lnSecurityLabelValueBegin  = AT( "=", lcSecurityLabelValue ) + 1
            lnSecurityLabelValueEnd    = AT( CHR(13), lcSecurityLabelValue )
            lnSecurityLabelValueLength = lnSecurityLabelValueEnd - ;
                                         lnSecurityLabelValueBegin
            lcSecurityLabel            = SUBSTR( lcSecurityLabelValue, ;
                                                 lnSecurityLabelValueBegin, ;
                                                 lnSecurityLabelValueLength )
            lcSecurityLabel = PADR( ALLTRIM( lcSecurityLabel ), 70 )
      
            *-----------------------------------------------------------------------
            *-- If no Security Label has been assigned to this control, skip over it
            *-- and move to the next one.
            *-----------------------------------------------------------------------
            IF EMPTY( lcSecurityLabel ) OR "CSECURITYLABEL" $ UPPER( lcSecurityLabel )
               LOOP
            ENDIF
      
            *-----------------------------------------------
            *-- Extract the description of the secure object
            *-----------------------------------------------
            lnDescriptionPropertyNameStart = AT( UPPER( "cDescriptiveName" ), ;
                                                 UPPER( m.properties ) )
            lcDescriptionValue             = SUBSTR( m.properties, ;
                                                     lnDescriptionPropertyNameStart, 87)
            IF .NOT. EMPTY( lcDescriptionValue )
               lnDescriptionValueBegin    = AT( "=", lcDescriptionValue ) + 1
               lnDescriptionValueEnd      = AT( CHR(13), lcDescriptionValue )
               lnDescriptionValueLength   = lnDescriptionValueEnd - lnDescriptionValueBegin
               lcDescriptionValue         = SUBSTR( lcDescriptionValue, ;
                                                    lnDescriptionValueBegin, ;
                                                    lnDescriptionValueLength )
               lcDescriptionValue    = PADR( ALLTRIM( lcDescriptionValue ), 70 )
            ELSE
               lcDescriptionValue    = PADR("No Description Provided",70)
            ENDIF
   
            *-----------------------------
            *-- Get the name of the object
            *-----------------------------
            lcInterfaceObjectName = PADR( ALLTRIM( m.ObjName ), 70 )

            *------------------------------------------
            *-- Encrypt the security object information      
            *------------------------------------------
            lcEncryptedSecurityLabel       = ""
            lcEncryptedInterfaceObjectName = ""
            lcEncryptedDescriptionValue    = ""
            oString.Encrypt( lcSecurityLabel       , @lcEncryptedSecurityLabel )
            oString.Encrypt( lcInterfaceObjectName , @lcEncryptedInterfaceObjectName )
            oString.Encrypt( lcDescriptionValue    , @lcEncryptedDescriptionValue )

            *------------------------------------------------
            *-- If the security object does not exist, add it
            *------------------------------------------------
            IF .NOT. SEEK( lcEncryptedSecurityLabel, "table2", "seclabel")
               INSERT INTO table2 ( field1                         , ;
                                    field2                         , ;
                                    field3                         ) ;
                           VALUES ( lcEncryptedSecurityLabel       , ;
                                    lcEncryptedInterfaceObjectName , ;
                                    lcEncryptedDescriptionValue      )
            ELSE
               *-- WAIT WINDOW lcSecurityLabel + " is already present in table."
            ENDIF
         ELSE
            LOOP
         ENDIF
      ENDFOR && -lnCount = 1 to lnNumberOfSecurityLabels
   ENDSCAN

   IF USED('id')
      USE IN id
   ENDIF

   IF USED('table2')
      USE IN table2
   ENDIF

   IF USED('controls')
      USE IN controls 
   ENDIF

   SET PATH TO &lcOldPath
   SELECT ( lnOldWA )

ENDFUNC

*=====================================
FUNCTION ExtractReportSecurityLabels()
*=====================================
   LOCAL lcOldPath, ;
         lnOldWA
         
   lcOldPath = SET('PATH')
   SET PATH TO security
   lnOldWA = SELECT()
   
   SELECT 0
   USE metadata\repolist ALIAS reports

   SELECT 0
   USE security\table2

   SELECT reports

   SET CLASSLIB TO COMMONPATH\libs\cdata.vcx
   oString = CREATEOBJ('cstring')

   *----------------------------------------------------------------
   *-- Scan through each control in the ACNTRL.VCX looking for class 
   *-- definitions that define a security control.  A security control
   *-- is defined as any control THAT HAS A VALUE placed into its
   *-- ==> "cSecurityLabel" <== property.
   *-----------------------------------------------------------------
   SCAN FOR .NOT. EMPTY( reports.cseclabel )

      lcSecurityLabel       = reports.cSecLabel
      lcInterfaceObjectName = reports.cFullName      
      lcDescriptionValue    = SYS(3) + SYS(3) + SYS(3)
      
      *------------------------------------------
      *-- Encrypt the security object information      
      *------------------------------------------
      lcEncryptedSecurityLabel       = ""
      lcEncryptedInterfaceObjectName = ""
      lcEncryptedDescriptionValue    = ""

      oString.Encrypt( lcSecurityLabel       , @lcEncryptedSecurityLabel )
      oString.Encrypt( lcInterfaceObjectName , @lcEncryptedInterfaceObjectName )
      oString.Encrypt( lcDescriptionValue    , @lcEncryptedDescriptionValue )

      *------------------------------------------------
      *-- If the security object does not exist, add it
      *------------------------------------------------
      IF .NOT. SEEK( lcEncryptedSecurityLabel, "table2", "seclabel")
         INSERT INTO table2 ( field1                         , ;
                              field2                         , ;
                              field3                         ) ;
                     VALUES ( lcEncryptedSecurityLabel       , ;
                              lcEncryptedInterfaceObjectName , ;
                              lcEncryptedDescriptionValue      )
      ELSE
         *-- WAIT WINDOW lcSecurityLabel + " is already present in table."
      ENDIF
   ENDSCAN

   IF USED('table2')
      USE IN table2
   ENDIF

   IF USED('reports')
      USE IN reports
   ENDIF

   SET PATH TO &lcOldPath
   SELECT ( lnOldWA )

ENDFUNC

*==================================
FUNCTION SychronizeSecurityTables()
*==================================
   LOCAL llRetVal, ;
         laUserIDList, ;
         llVisible, ;
         llEditable 

   llRetVal   = .T.
   DIMENSION laUserIDList[1]
   llVisible  = .F.
   llEditable = .F.
   
   IF .NOT. DBUSED('security')
      OPEN DATABASE security
   ENDIF
   
   SET DATABASE TO security
   
   llRetVal = GetListOfCurrentlyDefinedUserIDs( @laUserIDList )
   llRetVal = llRetVal AND ;
              GetApplicationSecurityDefaults( @llVisible  , ;
                                              @llEditable   )
   llRetVal = llRetVal AND ;
              UpdateInterfaceObjectSecurityTable( @laUserIDList , ;
                                                   llVisible    , ;
                                                   llEditable     )
   
   SET DATABASE TO

   RETURN llRetVal   
ENDFUNC

*========================================================
FUNCTION GetListOfCurrentlyDefinedUserIDs( taUserIDList )
*========================================================
LOCAL llRetVal

   SELECT DISTINCT field1            ;
              FROM table1            ;
              WHERE .NOT. DELETED()  ; 
              INTO ARRAY taUserIDList
              
   lnNoOfUsersWithSecurityDefined = _TALLY
   
   llRetVal = ( lnNoOfUsersWithSecurityDefined > 0 )
   
   IF USED('table1')
      USE IN table1
   ENDIF
   
   RETURN llRetVal
ENDFUNC

*===============================================================
FUNCTION GetApplicationSecurityDefaults( tlVisible, tlEditable )
*===============================================================
LOCAL llRetVal
llRetVal = .T.

   tlVisible  = 0
   tlEditable = 0

   SELECT 0
   USE table5
   
   tlVisible  = table5.field3
   tlEditable = table5.field2
   
   IF USED('table5')
      USE IN table5
   ENDIF
   
   llRetVal = TYPE( 'tlVisible' )  = "L" AND ;
              TYPE( 'tlEditable' ) = "L"
              
RETURN llRetVal

ENDFUNC

*==================================================================================
FUNCTION UpdateInterfaceObjectSecurityTable( taUserIDList, tlVisible , tlEditable )
*==================================================================================
   LOCAL lcUserID                  , ;
         lcSecurityObjectLabel     , ;
         lcEncryptedFieldValue     , ;
         lcUserSecurityObjectID    , ;
         lnNumberOfRecordsInserted , ;
         lnNumberOfUsersWithSecurityDefined
         
   SELECT 0
   USE table1   &&-- Interface Object Security Table

   SELECT 0
   USE table2   &&-- Interface Object Security Label Table   
   
   lnNumberOfUsersWithSecurityDefined = ALEN( taUserIDList, 1 )
   
   *--------------------------------------------------------
   *-- Scan through each security label and match it up with
   *-- one of the defined users, then, determine if this 
   *-- security object has already been associated with this
   *-- user.  If it has, don't insert it into the 
   *-- Interface Object Security Table.
   *--------------------------------------------------------
   lnNumberOfRecordsInserted = 0
   SCAN
      FOR lnUser = 1 TO lnNumberOfUsersWithSecurityDefined
      
         lcUserID                = taUserIDList( lnUser )
         lcSecurityObjectLabel   = table2.field1
         lcUserSecurityObjectID  = lcUserID + table2.field1
         
         IF .NOT. SEEK( lcUserSecurityObjectID, 'table1', 'UserObjSec' )
            lnNumberOfRecordsInserted = lnNumberOfRecordsInserted + 1
            INSERT INTO table1 ( field1               , ;
                                 field2               ) ;
                         VALUES( lcUserID             , ;
                                 lcSecurityObjectLabel  )

            lcEncryptedFieldValue = ""
            
            *===================================================
            *== Field 4 is the place where the logical value for 
            *== VISIBLE is stored, encrypt it and populate that
            *== value with the application default setup by the
            *== application administrator
            *===================================================
            IF tlVisible 
               =GetEncryptedLogical( "TRUE" , "6", 6, @lcEncryptedFieldValue )
            ELSE
               =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            ENDIF               
            REPLACE table1.field4 WITH lcEncryptedFieldValue
            
            
            *===================================================
            *== Field 7 is the place where the logical value for 
            *== EDITABLE is stored, encrypt it and populate that
            *== value with the application default setup by the
            *== application administrator
            *===================================================
            IF tlEditable 
               =GetEncryptedLogical( "TRUE" , "6", 6, @lcEncryptedFieldValue )
            ELSE
               =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            ENDIF               
            REPLACE table1.field7 WITH lcEncryptedFieldValue

            *===========================================================
            *== Field 3, 5, 6, 8, and 9 are confusion fields designed to
            *== conceal the real information that is encrypted in fields
            *== four and seven.  Populate them with random data
            *===========================================================
            =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            REPLACE table1.field3 WITH lcEncryptedFieldValue

            =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            REPLACE table1.field5 WITH lcEncryptedFieldValue

            =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            REPLACE table1.field6 WITH lcEncryptedFieldValue

            =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            REPLACE table1.field8 WITH lcEncryptedFieldValue

            =GetEncryptedLogical( "FALSE", "6", 6, @lcEncryptedFieldValue )
            REPLACE table1.field9 WITH lcEncryptedFieldValue
         ENDIF
      ENDFOR
   ENDSCAN   
   
   IF lnNumberOfRecordsInserted = 0
      WAIT WINDOW " No new user security information created ... " TIMEOUT 2
   ELSE
      lcNumberOfRecordsInserted = ALLTRIM(STR( lnNumberOfRecordsInserted ))
      WAIT WINDOW lcNumberOfRecordsInserted + ;
                  " new user interface object records created !!!" TIMEOUT 5
   ENDIF
   
   IF USED('table1')
      USE IN table1
   ENDIF
   
   IF USED('table2')
      USE IN table2
   ENDIF
   
ENDFUNC

*=================================
FUNCTION RemoveBadSecurityLabels()
*=================================
   LOCAL lcOldPath, ;
         lcOldError, ;
         lnOldWA, ;
         llRetVal, ;
         lcSecuritylabel, ;
         lcEncryptedSecurityLabel, ;
         lcLabelsDeleted, ;
         lcUILabelsDeleted, ;
         lcMessage
         
   lcOldError = ON('ERROR')
   ON ERROR llRetVal = .F.      
   llRetVal = .T.
   
   lcOldPath = SET('PATH')
   SET PATH TO security
   lnOldWA = SELECT()
   
   SELECT 0
   USE security\table2 EXCL

	IF llRetVal
		SELECT 0
		USE security\table1 EXCL
	ENDIF
	
	IF llRetVal

		SET CLASSLIB TO COMMONPATH\libs\cdata.vcx
		oString = CREATEOBJ('cstring')

		*--------------------------------
		*-- Delete the bad security label
		*--------------------------------
		IF llRetVal
			*------------------------------------
			*-- Clean up the security label table
			*------------------------------------
			lnDeleted = 0
			SELECT table2
			SCAN
				IF oString.Decrypt( table2.field1, @lcSecurityLabel )
					IF "CSECURITYLABEL" $ UPPER( lcSecurityLabel )
						DELETE
						lnDeleted = lnDeleted + 1
					ENDIF
				ENDIF
			ENDSCAN
			
			lcLabelsDeleted = ALLTRIM( TRANSFORM( lnDeleted, "999,999" ) )

			*----------------------------------------------------
			*-- Clean up the user interface object security table
			*----------------------------------------------------
			lnDeleted = 0
			SELECT table1
			SCAN
				IF oString.Decrypt( table1.field2, @lcSecurityLabel )
					IF "CSECURITYLABEL" $ UPPER( lcSecurityLabel )
						DELETE
						lnDeleted = lnDeleted + 1
					ENDIF
				ENDIF
			ENDSCAN

			lcUILabelsDeleted = ALLTRIM( TRANSFORM( lnDeleted, "999,999" ) )
			
			lcMessage = lcLabelsDeleted + " security label(s) deleted." + CHR(13) + CHR(13) + ;
							lcUILabelsDeleted + " user interface object security permission(s) deleted."

			MESSAGEBOX( lcMessage )				
		ENDIF
	ELSE
		MESSAGEBOX( "Someone is still using the system.  " + ;
						"Please get everyone to log out of your " + ;
						"application before running this process." )
	ENDIF
	
	*----------------------------------------------------
	*-- Pack the tables to get rid of the deleted records
	*----------------------------------------------------
	IF llRetval 
		SELECT table1
		PACK
	
		SELECT table2
		PACK
	ENDIF

	IF USED('table1')
		USE IN table1
	ENDIF

	IF USED('table2')
		USE IN table2
	ENDIF
		
	SET PATH TO &lcOldPath
	ON ERROR &lcOldError
	SELECT (lnOldWA)

	RETURN llRetVal
ENDFUNC

